"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const prettier_1 = __importDefault(require("prettier"));
const fast_diff_1 = __importDefault(require("fast-diff"));
function minimalEdits(original, formatted) {
    const diffs = (0, fast_diff_1.default)(original, formatted);
    let pos = 0;
    const changes = [];
    for (const [type, text] of diffs) {
        switch (type) {
            case fast_diff_1.default.INSERT: {
                changes.push({
                    span: {
                        start: pos,
                        length: 0,
                    },
                    newText: text,
                });
                break;
            }
            case fast_diff_1.default.DELETE: {
                changes.push({
                    span: {
                        start: pos,
                        length: text.length,
                    },
                    newText: '',
                });
                pos += text.length;
                break;
            }
            case fast_diff_1.default.EQUAL: {
                pos += text.length;
                break;
            }
        }
    }
    return changes;
}
function format(filePath, content, start, end, log) {
    const config = prettier_1.default.resolveConfig.sync(filePath, {
        useCache: true,
        editorconfig: true,
    });
    log(`formatting config: ${JSON.stringify(config)}`);
    const formatted = prettier_1.default.format(content, Object.assign(Object.assign({}, config), { rangeStart: start, rangeEnd: end, filepath: filePath }));
    const changes = minimalEdits(content, formatted);
    log(`original document:\n${content}\n\nformatted document\n${formatted}\n\nminimal edits:\n${JSON.stringify(changes, null, 2)}\n\n`);
    return changes;
}
function getFileContent(info, filePath) {
    var _a, _b;
    const src = (_b = (_a = info.languageService
        .getProgram()) === null || _a === void 0 ? void 0 : _a.getSourceFile(filePath)) === null || _b === void 0 ? void 0 : _b.getFullText();
    if (src == null) {
        throw new Error('Failed to get file');
    }
    return src;
}
function create(info) {
    const log = (msg) => info.project.projectService.logger.info(msg);
    log('Prettier plugin starting up');
    const proxy = Object.create(null);
    for (let k of Object.keys(info.languageService)) {
        const x = info.languageService[k];
        // @ts-expect-error
        proxy[k] = (...args) => x.apply(info.languageService, args);
    }
    proxy.getFormattingEditsForDocument = (filePath, _options) => {
        log(`getFormattingEditsForDocument ${filePath}`);
        const content = getFileContent(info, filePath);
        return format(filePath, content, 0, content.length, log);
    };
    proxy.getFormattingEditsForRange = (filePath, start, end, _options) => {
        log(`getFormattingEditsForRange ${filePath} range: ${start} - ${end}`);
        return format(filePath, getFileContent(info, filePath), start, end, log);
    };
    proxy.getFormattingEditsAfterKeystroke = (filePath, _position, _key, _options) => {
        log(`getFormattingEditsAfterKeystroke ${filePath} not implemented`);
        throw new Error('Not implemented');
    };
    return proxy;
}
function factory(_modules) {
    return { create };
}
module.exports = factory;
//# sourceMappingURL=index.js.map