import prettier from 'prettier';
import fastDiff from 'fast-diff';
import type ts from 'typescript/lib/tsserverlibrary';

function minimalEdits(original: string, formatted: string): ts.TextChange[] {
  const diffs = fastDiff(original, formatted);

  let pos = 0;
  const changes: ts.TextChange[] = [];
  for (const [type, text] of diffs) {
    switch (type) {
      case fastDiff.INSERT: {
        changes.push({
          span: {
            start: pos,
            length: 0,
          },
          newText: text,
        });

        break;
      }

      case fastDiff.DELETE: {
        changes.push({
          span: {
            start: pos,
            length: text.length,
          },
          newText: '',
        });
        pos += text.length;

        break;
      }

      case fastDiff.EQUAL: {
        pos += text.length;

        break;
      }
    }
  }

  return changes;
}

function format(
  filePath: string,
  content: string,
  start: number,
  end: number,
  log: (msg: string) => void,
): ts.TextChange[] {
  const config = prettier.resolveConfig.sync(filePath, {
    useCache: true,
    editorconfig: true,
  });

  log(`formatting config: ${JSON.stringify(config)}`);

  const formatted = prettier.format(content, {
    ...config, // TODO merge with tsOptions
    rangeStart: start,
    rangeEnd: end,
    filepath: filePath,
  });

  const changes = minimalEdits(content, formatted);

  log(
    `original document:\n${content}\n\nformatted document\n${formatted}\n\nminimal edits:\n${JSON.stringify(
      changes,
      null,
      2,
    )}\n\n`,
  );

  return changes;
}

function getFileContent(info: ts.server.PluginCreateInfo, filePath: string) {
  const src = info.languageService
    .getProgram()
    ?.getSourceFile(filePath)
    ?.getFullText();

  if (src == null) {
    throw new Error('Failed to get file');
  }

  return src;
}

function create(info: ts.server.PluginCreateInfo) {
  const log = (msg: string) => info.project.projectService.logger.info(msg);

  log('Prettier plugin starting up');

  const proxy: ts.LanguageService = Object.create(null);
  for (let k of Object.keys(info.languageService) as Array<
    keyof ts.LanguageService
  >) {
    const x = info.languageService[k]!;
    // @ts-expect-error
    proxy[k] = (...args: Array<{}>) => x.apply(info.languageService, args);
  }

  proxy.getFormattingEditsForDocument = (
    filePath: string,
    _options: ts.FormatCodeOptions | ts.FormatCodeSettings,
  ): ts.TextChange[] => {
    log(`getFormattingEditsForDocument ${filePath}`);

    const content = getFileContent(info, filePath);

    return format(filePath, content, 0, content.length, log);
  };

  proxy.getFormattingEditsForRange = (
    filePath: string,
    start: number,
    end: number,
    _options: ts.FormatCodeOptions | ts.FormatCodeSettings,
  ): ts.TextChange[] => {
    log(`getFormattingEditsForRange ${filePath} range: ${start} - ${end}`);

    return format(filePath, getFileContent(info, filePath), start, end, log);
  };

  proxy.getFormattingEditsAfterKeystroke = (
    filePath: string,
    _position: number,
    _key: string,
    _options: ts.FormatCodeOptions | ts.FormatCodeSettings,
  ): ts.TextChange[] => {
    log(`getFormattingEditsAfterKeystroke ${filePath} not implemented`);

    throw new Error('Not implemented');
  };

  return proxy;
}

function factory(_modules: { typescript: typeof ts }) {
  return { create };
}

export = factory;

